<?php

namespace Drupal\a11n_code_example\Plugin\AuthorizationMethod;

use Drupal\authorization_code\AccountIdentifierManager;
use Drupal\authorization_code\AuthorizationCodeRepository;
use Drupal\authorization_code\Plugin\AuthorizationMethodBase;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\TypedData\TypedDataManagerInterface;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * An authorization method for debug purposes only.
 *
 * When using this method, the authorization code will be displayed as a status
 * message using the Messages API.
 *
 * @AuthorizationMethod(
 *   id = "echo",
 *   label = @Translation("Echo"),
 *   debug = true,
 *   argument_type = "integer",
 *   identifiers = {"unique_user_field:uid", "unique_user_field:name", "unique_user_field:mail"}
 * )
 */
class EchoAuthorization extends AuthorizationMethodBase {

  /**
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * EchoAuthorization constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\TypedData\TypedDataManagerInterface $typed_data_manager
   *   The typed data manager.
   * @param \Drupal\authorization_code\AuthorizationCodeRepository $repository
   *   The authorization code repository.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   * @param \Drupal\authorization_code\AccountIdentifierManager $account_identifier_manager
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, TypedDataManagerInterface $typed_data_manager, AuthorizationCodeRepository $repository, MessengerInterface $messenger, AccountIdentifierManager $account_identifier_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $typed_data_manager, $repository, $account_identifier_manager);
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('typed_data_manager'),
      $container->get('authorization_code.repository'),
      $container->get('messenger'),
      $container->get('plugin.manager.account_identifier')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function doSendCode($argument, $code) {
    $this->messenger->addStatus($this->t('The code for user with id @arg, is @code', [
      '@arg' => $argument,
      '@code' => $code
    ]));
  }

  /**
   * {@inheritdoc}
   */
  public function loadUser($argument) {
    return User::load($argument);
  }

}
