<?php

namespace Drupal\a11n_code_example\Plugin\AuthorizationMethod;

use Drupal\authorization_code\AccountIdentifierManager;
use Drupal\authorization_code\AuthorizationCodeRepository;
use Drupal\authorization_code\Plugin\AuthorizationMethodBase;
use Drupal\Core\Logger\LoggerChannel;
use Drupal\Core\TypedData\TypedDataManagerInterface;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * An authorization method for debug purposes only.
 *
 * When using this method, the authorization code will be logged, but not printed.
 *
 * @AuthorizationMethod(
 *   id = "watchdog",
 *   label = @Translation("Watchdog"),
 *   debug = true,
 *   argument_type = "integer",
 *   identifiers = {"unique_user_field:uid", "unique_user_field:name", "unique_user_field:mail"}
 * )
 */
class WatchdogAuthorization extends AuthorizationMethodBase {

  /**
   * @var \Drupal\Core\Logger\LoggerChannel
   */
  protected $logger;

  /**
   * WatchdogAuthorization constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\TypedData\TypedDataManagerInterface $typed_data_manager
   *   The typed data manager.
   * @param \Drupal\authorization_code\AuthorizationCodeRepository $repository
   *   The authorization code repository.
   * @param \Drupal\Core\Logger\LoggerChannel $logger
   * @param \Drupal\authorization_code\AccountIdentifierManager $account_identifier_manager
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, TypedDataManagerInterface $typed_data_manager, AuthorizationCodeRepository $repository, LoggerChannel $logger, AccountIdentifierManager $account_identifier_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $typed_data_manager, $repository, $account_identifier_manager);
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('typed_data_manager'),
      $container->get('authorization_code.repository'),
      $container->get('logger.channel.authorization_code'),
      $container->get('plugin.manager.account_identifier')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function doSendCode($argument, $code) {
    $this->logger->debug('The code for user with id @arg, is @code', [
      '@arg' => $argument,
      '@code' => $code
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function loadUser($argument) {
    return User::load($argument);
  }

}
