<?php

namespace Drupal\a11n_code_rest\Authentication\Provider;

use Drupal\authorization_code\AuthorizationCodeRepository;
use Drupal\authorization_code\AuthorizationMethodManager;
use Drupal\Core\Authentication\AuthenticationProviderInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Session\Session;
use Symfony\Component\Serializer\Serializer;

/**
 * Authentication service that uses the authorization_code session parameter set
 * by the authorization_code:authorize resource.
 *
 * @see \Drupal\a11n_code_rest\Plugin\rest\resource\AuthorizeCodeResource
 */
class AuthorizationCode implements AuthenticationProviderInterface, ContainerInjectionInterface {

  /**
   * @var \Symfony\Component\Serializer\Serializer
   */
  protected $serializer;

  /**
   * @var \Symfony\Component\HttpFoundation\Session\Session
   */
  protected $session;

  /**
   * @var \Drupal\authorization_code\AuthorizationMethodManager
   */
  protected $manager;

  /**
   * @var \Drupal\authorization_code\AuthorizationCodeRepository
   */
  protected $repository;

  /**
   * AuthorizationCode constructor.
   * @param \Symfony\Component\Serializer\Serializer $serializer
   * @param \Symfony\Component\HttpFoundation\Session\Session $session
   * @param \Drupal\authorization_code\AuthorizationMethodManager $manager
   *   The authorization method manager.
   * @param \Drupal\authorization_code\AuthorizationCodeRepository $repository
   *   The authorization code repository.
   */
  public function __construct(Serializer $serializer, Session $session, AuthorizationMethodManager $manager, AuthorizationCodeRepository $repository) {
    $this->serializer = $serializer;
    $this->session = $session;
    $this->repository = $repository;
    $this->manager = $manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('serializer'),
      $container->get('session'),
      $container->get('plugin.manager.authorization_method'),
      $container->get('authorization_code.repository')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function applies(Request $request) {
    $session_data = $this->session->get('authorization_code');
    return !empty($session_data['authorized']);
  }

  /**
   * {@inheritdoc}
   */
  public function authenticate(Request $request) {
    if ($session_data = $this->session->get('authorization_code')) {
      return isset($session_data['user']) ? $session_data['user'] : NULL;
    }

    return NULL;
  }

}
