<?php

namespace Drupal\authorization_code;

use Drupal\authorization_code\Annotation\AccountIdentifier;
use Drupal\authorization_code\Plugin\AccountIdentifierInterface;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;

/**
 * Manages the account identifier plugins.
 */
class AccountIdentifierManager extends DefaultPluginManager {

  /**
   * @var \Drupal\authorization_code\Plugin\AccountIdentifierInterface[]
   */
  protected $plugins = [];

  /**
   * AccountIdentifierManager constructor.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct('Plugin/AccountIdentifier', $namespaces, $module_handler, AccountIdentifierInterface::class, AccountIdentifier::class);
    $this->setCacheBackend($cache_backend, 'account_identifier_plugins');
    $this->alterInfo('account_identifier');
  }

  /**
   * @param array $options
   *
   * @return false|\Drupal\authorization_code\Plugin\AccountIdentifierInterface
   */
  public function getInstance(array $options) {
    if (isset($options['id'])) {
      $plugin_id = $options['id'];

      if (empty($this->plugins[$plugin_id])) {
        try {
          $this->plugins[$plugin_id] = $this->createInstance($plugin_id);
        }
        catch (PluginNotFoundException $e) {
          return FALSE;
        }
      }

      return $this->plugins[$plugin_id];
    }

    return FALSE;
  }

  /**
   * @param string $plugin_id
   *
   * @return false|\Drupal\authorization_code\Plugin\AccountIdentifierInterface
   */
  public function getPlugin($plugin_id) {
    return $this->getInstance(['id' => $plugin_id]);
  }

  /**
   * {@inheritdoc}
   *
   * @return \Drupal\authorization_code\Plugin\AccountIdentifierInterface
   */
  public function createInstance($plugin_id, array $configuration = []) {
    return parent::createInstance($plugin_id, $configuration);
  }

}
