<?php

namespace Drupal\authorization_code\Annotation;

use Drupal\Component\Annotation\Plugin;

/**
 * Defines the AuthorizationMethod annotation object.
 *
 * Authorization methods are used to send authorization codes to users that want
 * to login or register.
 *
 * Plugin namespace: Plugin\AuthorizationMethod
 *
 * @see \Drupal\authorization_code\Plugin\AuthorizationMethodInterface
 * @see \Drupal\authorization_code\Plugin\AuthorizationMethodBase
 * @see \Drupal\authorization_code\AuthorizationMethodManager
 * @see plugin_api
 *
 * @Annotation
 */
class AuthorizationMethod extends Plugin {

  /**
   * The machine name of this plugin.
   *
   * @var string
   */
  public $id;

  /**
   * The display name of this plugin.
   *
   * @var string
   */
  public $label;

  /**
   * Used to indicate that this authorization method will only be used if debug
   * mode is enabled.
   *
   * @var bool
   */
  public $debug = FALSE;

  /**
   * A list of all the supported account identifiers.
   *
   * @var string[]
   */
  public $identifiers;

  // @codingStandardsIgnoreStart
  /**
   * The typed data definition of the received argument.
   *
   * @var string|\Drupal\Core\TypedData\DataDefinitionInterface
   */
  public $argument_type = 'any';
  // @codingStandardsIgnoreEnd

  /**
   * {@inheritdoc}
   */
  public function __construct($values) {
    $values += [
      'argument_type' => 'any',
      'debug' => FALSE,
    ];
    if (is_string($values['argument_type'])) {
      $values['argument_type'] = \Drupal::typedDataManager()
        ->createDataDefinition($values['argument_type']);
    }
    parent::__construct($values);
  }

}
