<?php

namespace Drupal\authorization_code\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form settings class for the authorization code module.
 */
class AuthorizationCodeSettingsForm extends ConfigFormBase {

  /**
   * @var \Drupal\Core\Config\TypedConfigManagerInterface
   */
  protected $typedConfigManager;

  /**
   * AuthorizationCodeSettingsForm constructor.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typed_config_manager
   */
  public function __construct(ConfigFactoryInterface $config_factory, TypedConfigManagerInterface $typed_config_manager) {
    parent::__construct($config_factory);
    $this->typedConfigManager = $typed_config_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed')
    );
  }

  /**
   * @return \Drupal\Core\Config\Config
   */
  protected function getSettingsConfig() {
    return $this->config('authorization_code.settings');
  }

  /**
   * @return \Drupal\Core\TypedData\MapDataDefinition
   */
  protected function getSettingsSchema() {
    return $this->typedConfigManager->get('authorization_code.settings')
      ->getDataDefinition();
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $form['code_length'] = [
      '#type' => 'textfield',
      '#size' => 2,
      '#maxlength' => 2,
      '#required' => TRUE,
    ];

    $form['flood_threshold'] = [
      '#type' => 'textfield',
      '#size' => 2,
      '#maxlength' => 2,
      '#required' => TRUE,
    ];

    $form['flood_window'] = [
      '#type' => 'textfield',
      '#size' => 4,
      '#maxlength' => 4,
      '#required' => TRUE,
    ];

    $form['daily_code_limit'] = [
      '#type' => 'textfield',
      '#size' => 2,
      '#maxlength' => 2,
      '#required' => TRUE,
    ];

    $form['max_fetch_count'] = [
      '#type' => 'textfield',
      '#size' => 2,
      '#maxlength' => 2,
      '#required' => TRUE,
    ];

    $form['code_expiration_time'] = [
      '#type' => 'textfield',
      '#size' => 5,
      '#maxlength' => 5,
      '#required' => TRUE,
    ];

    $form['debug_mode_enabled'] = [
      '#type' => 'checkbox',
    ];

    $config = $this->getSettingsConfig();
    $schema = $this->getSettingsSchema();
    //$schema['mapping']['code_length']
    foreach (Element::children($form) as $key) {
      if ($default_value = $config->get($key)) {
        $form[$key]['#default_value'] = $default_value;
      }

      if (isset($schema['mapping'][$key]['label'])) {
        $form[$key]['#title'] = $schema['mapping'][$key]['label'];
      }

      if (isset($schema['mapping'][$key]['description'])) {
        $form[$key]['#description'] = $schema['mapping'][$key]['description'];
      }
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->getSettingsConfig();
    $schema = $this->getSettingsSchema();
    foreach ($form_state->getValues() as $key => $value) {
      if (isset($schema['mapping'][$key])) {
        $config->set($key, $value);
      }
    }

    $config->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['authorization_code.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'authorization_code_settings';
  }

}
