<?php

namespace Drupal\authorization_code\Plugin\AccountIdentifier;

use Drupal\authorization_code\Plugin\AccountIdentifierBase;

/**
 * @AccountIdentifier(
 *   id = "unique_user_field",
 *   label = @Translation("Unique user field"),
 *   deriver = "Drupal\authorization_code\Plugin\Derivative\UniqueUserField"
 * )
 */
class UniqueUserField extends AccountIdentifierBase {

  /**
   * {@inheritdoc}
   */
  public function getUserId($identifier) {
    $results = $this->userStorage->getQuery()
      ->condition($this->getPropertyPath(), $identifier, '=')
      ->execute();

    switch (count($results)) {
      case 0:
        return NULL;
      case 1:
        return reset($results);
      default:
        throw new \LogicException('More than 1 user were found with the same unique identifier.');
    }
  }

  /**
   * Concatenates the field name with the main property name to arrive at the
   * complete property path.
   *
   * @return string
   *   The property path.
   */
  protected function getPropertyPath() {
    return implode('.', [
      $this->getPluginDefinition()['field_name'],
      $this->getPluginDefinition()['main_property_name'],
    ]);
  }

}
