<?php

namespace Drupal\authorization_code\Plugin;

use Drupal\Component\Plugin\PluginInspectionInterface;

/**
 * Defines the required interface for all authorization method plugins.
 */
interface AuthorizationMethodInterface extends PluginInspectionInterface {

  /**
   * @param mixed $identifier
   *   The account identifier.
   *
   * @throws \Drupal\authorization_code\Exception\SendCodeException
   */
  public function sendCode($identifier);

  /**
   * @param mixed $identifier
   *   The account identifier.
   * @param string $code
   *   The code to test against the on in the repository.
   * @return bool
   *   True if the code is valid, false otherwise.
   *
   * @see \Drupal\authorization_code\AuthorizationCodeRepository::verifyCode()
   */
  public function verifyCode($identifier, $code);

  /**
   * @param mixed $identifier
   *   The account identifier.
   * @return bool
   */
  public function isValidIdentifier($identifier);

  /**
   * @param mixed $identifier
   *   The account identifier.
   * @return \Symfony\Component\Validator\ConstraintViolationListInterface
   */
  public function validateIdentifier($identifier);

  /**
   * Loads the user object from the provided identifier.
   *
   * @param mixed $identifier
   *   The account identifier.
   *
   * @return null|\Drupal\user\UserInterface
   *   The fully loaded user object, or null if no user was matched to the
   *   identifier.
   */
  public function loadUser($identifier);

}
