<?php

namespace Drupal\Tests\authorization_code\Kernel\Plugin\AccountIdentifier;

use Drupal\authorization_code\Plugin\AccountIdentifierInterface;
use Drupal\Tests\authorization_code\Kernel\Plugin\PluginKernelTestBase;

/**
 * Base class for account identifier plugins.
 */
abstract class AccountIdentifierTestBase extends PluginKernelTestBase {

  /**
   * @var string[]
   */
  protected static $fieldNames = ['uid', 'name', 'mail'];

  /**
   * @var \Drupal\user\UserInterface[]
   */
  protected $users;

  protected static $pluginInterface = AccountIdentifierInterface::class;

  /**
   * @var string
   * @see \Drupal\Tests\authorization_code\Kernel\Plugin\PluginKernelTestBase::setUp()
   */
  protected static $pluginName = 'account_identifier';

  /**
   * @var \Drupal\authorization_code\AccountIdentifierManager
   * @see \Drupal\Tests\authorization_code\Kernel\Plugin\PluginKernelTestBase::setUp()
   */
  protected $manager;

  /**
   * @param string $field_name
   * @param string $plugin_name
   * @param array $user_data
   *
   * @dataProvider userDataProvider
   */
  public function testUserFetch($field_name, $plugin_name, array $user_data) {
    $user = $this->createUser($user_data);
    $this->createDummyUsers($user->id() + 1);
    $plugin = $this->manager->createInstance($plugin_name);
    $actual_uid = $plugin->getUserId($user_data[$field_name]);
    $this->assertEquals($user->id(), $actual_uid, "The {$plugin_name} fetched the correct user id.");
  }

  /**
   * @param string $field_name
   * @param string $plugin_name
   * @param array $user_data
   *
   * @dataProvider userDataProvider
   */
  public function testUserLoad($field_name, $plugin_name, array $user_data) {
    $user = $this->createUser($user_data);
    $this->createDummyUsers($user->id() + 1);
    $plugin = $this->manager->createInstance($plugin_name);
    $actual_user = $plugin->loadUser($user_data[$field_name]);
    $this->assertEquals($user->toArray(), $actual_user->toArray(), "The {$plugin_name} loaded the correct user.");
  }

  /**
   * @param $plugin_name
   * @param $identifier
   * @param $isValid
   *
   * @dataProvider validationProvider
   */
  public function testIsValid($plugin_name, $identifier, $isValid) {
    $plugin = $this->manager->createInstance($plugin_name);
    $this->assertEquals($isValid, $plugin->isValid($identifier), "The isValid method of {$plugin_name} works as expected.");
  }

  /**
   * @return array[]
   */
  abstract public function validationProvider();

  /**
   * @return array[]
   */
  abstract public function userDataProvider();

}
