<?php

namespace Drupal\Tests\authorization_code\Kernel\Plugin\AccountIdentifier;

/**
 * Confirm that the unique_user_field plugin works properly
 *
 * @group authorization_code
 */
class UniqueUserField extends AccountIdentifierTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $fieldNames = ['uid', 'name', 'mail'];

  /**
   * {@inheritdoc}
   */
  public function validationProvider() {
    return [
      'uid:positive' => [
        'unique_user_field:uid',
        1,
        TRUE
      ],
      'uid:negative:string' => [
        'unique_user_field:uid',
        $this->randomMachineName(),
        FALSE
      ],
      'uid:negative:float' => [
        'unique_user_field:uid',
        2.5,
        FALSE
      ],
      'uid:negative:null' => [
        'unique_user_field:uid',
        NULL,
        FALSE
      ],
      'uid:negative:array' => [
        'unique_user_field:uid',
        [1],
        FALSE
      ],
      'uid:negative:object' => [
        'unique_user_field:uid',
        (object) (['a' => 1]),
        FALSE
      ],
      'name:positive' => [
        'unique_user_field:name',
        'Some user name',
        TRUE
      ],
      'name:negative:null' => [
        'unique_user_field:name',
        NULL,
        FALSE
      ],
      'name:negative:array' => [
        'unique_user_field:name',
        [1],
        FALSE
      ],
      'name:negative:object' => [
        'unique_user_field:name',
        (object) (['a' => 1]),
        FALSE
      ],
      'mail:positive' => [
        'unique_user_field:mail',
        'mail@example.com',
        TRUE
      ],
      'mail:negative:noSign' => [
        'unique_user_field:mail',
        'mailexample.com',
        FALSE
      ],
      'mail:negative:multiSign' => [
        'unique_user_field:mail',
        'multi@mail@example@com',
        FALSE
      ],
      'mail:negative:int' => [
        'unique_user_field:mail',
        9,
        FALSE
      ],
      'mail:negative:float' => [
        'unique_user_field:mail',
        7.3,
        FALSE
      ],
      'mail:negative:null' => [
        'unique_user_field:mail',
        NULL,
        FALSE
      ],
      'mail:negative:array' => [
        'unique_user_field:mail',
        [1],
        FALSE
      ],
      'mail:negative:object' => [
        'unique_user_field:mail',
        (object) (['a' => 1]),
        FALSE
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function pluginNamesProvider() {
    $plugin_names = [];
    foreach (static::$fieldNames as $field_name) {
      $plugin_names[$field_name] = ["unique_user_field:{$field_name}", [], TRUE];
    }
    foreach (['uuid', 'langcode', 'timezone', 'roles'] as $field_name) {
      $plugin_names[$field_name] = ["unique_user_field:{$field_name}", []];
    }
    return $plugin_names;
  }

  /**
   * {@inheritdoc}
   */
  public function userDataProvider() {
    return [
      'uid' => [
        'uid',
        'unique_user_field:uid',
        [
          'uid' => 2,
        ],
      ],
      'name' => [
        'name',
        'unique_user_field:name',
        [
          'name' => $this->randomMachineName(),
        ],
      ],
      'mail' => [
        'mail',
        'unique_user_field:mail',
        [
          'mail' => $this->randomMachineName() . '@example.com',
        ],
      ],
    ];
  }

}
