<?php

namespace Drupal\Tests\authorization_code\Kernel\Plugin;

use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\KernelTests\Core\Entity\EntityKernelTestBase;

/**
 * Base class for plugin test cases.
 */
abstract class PluginKernelTestBase extends EntityKernelTestBase {

  /**
   * Modules to enable.
   * @var array
   */
  public static $modules = ['authorization_code'];

  protected static $pluginInterface = PluginInspectionInterface::class;

  protected static $pluginName = NULL;

  /**
   * @var \Drupal\Core\Plugin\DefaultPluginManager
   */
  protected $manager;

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    $this->installConfig('authorization_code');

    if (is_string(static::$pluginName)) {
      $this->manager = $this->container->get('plugin.manager.' . static::$pluginName);
    }
  }

  /**
   * @param string $plugin_id
   * @param array $plugin_configuration
   * @param string|true $expected
   * @param string $exception_message
   *
   * @dataProvider pluginNamesProvider
   */
  public function testPluginExists($plugin_id, $plugin_configuration, $expected = PluginNotFoundException::class, $exception_message = NULL) {
    $exception_message = is_string($exception_message) ? $exception_message : "The \"{$plugin_id}\" plugin does not exist.";

    if ($expected === TRUE) {
      $this->assertInstanceOf(
        static::$pluginInterface,
        $this->manager->createInstance($plugin_id, $plugin_configuration),
        "The {$plugin_id} plugin exists."
      );
    }
    else {
      $this->setExpectedException($expected, $exception_message);
      $this->manager->createInstance($plugin_id, $plugin_configuration);
    }
  }

  /**
   * Creates dummy users to try and confuse the tests.
   * @param int $start_id
   * @param int $count
   */
  protected function createDummyUsers($start_id, $count = 10) {
    for ($i = 0; $i < $count; $i++) {
      $this->createUser(['uid' => $start_id + $i]);
    }
  }

  /**
   * @return array[]
   */
  abstract public function pluginNamesProvider();

}
